@@ -16,6 +16,8 @@ sys.path[0:0] = save_path
 import logging
 from six.moves import urllib
 import threading
+
+# from tests.utils.mock_requests import RequestsMock, InjectRequestsParse
 from yea_wandb.mock_requests import RequestsMock, InjectRequestsParse
 
 
@@ -30,11 +32,13 @@ def default_ctx():
         "current_run": None,
         "files": {},
         "k8s": False,
-        "resume": False,
+        "resume": None,
         "file_bytes": {},
         "manifests_created": [],
         "artifacts_by_id": {},
         "upsert_bucket_count": 0,
+        "max_cli_version": "0.10.33",
+        "runs": {},
     }
 
 
@@ -216,6 +220,12 @@ def get_ctx():
     return g.ctx.get()
 
 
+def get_run_ctx(run_id):
+    glob_ctx = get_ctx()
+    run_ctx = glob_ctx["runs"][run_id]
+    return run_ctx
+
+
 def set_ctx(ctx):
     get_ctx()
     g.ctx.set(ctx)
@@ -317,14 +327,6 @@ def create_app(user_ctx=None):
         app.logger.info("graphql post body: %s", body)
         if body["variables"].get("run"):
             ctx["current_run"] = body["variables"]["run"]
-        if "mutation UpsertBucket(" in body["query"]:
-            param_config = body["variables"].get("config")
-            if param_config:
-                ctx.setdefault("config", []).append(json.loads(param_config))
-            param_summary = body["variables"].get("summaryMetrics")
-            if param_summary:
-                ctx.setdefault("summary", []).append(json.loads(param_summary))
-            ctx["upsert_bucket_count"] += 1
 
         if body["variables"].get("files"):
             requested_file = body["variables"]["files"][0]
@@ -453,7 +455,14 @@ def create_app(user_ctx=None):
                             "teams": {
                                 "edges": []  # TODO make configurable for cli_test
                             },
-                        }
+                        },
+                        "serverInfo": {
+                            "cliVersionInfo": {
+                                "max_cli_version": str(
+                                    ctx.get("max_cli_version", "0.10.33")
+                                )
+                            }
+                        },
                     }
                 }
             )
@@ -524,19 +533,51 @@ def create_app(user_ctx=None):
                 }
             )
         if "mutation UpsertBucket(" in body["query"]:
+            run_id_default = "abc123"
+            run_id = body["variables"].get("name", run_id_default)
+            run_num = len(ctx["runs"])
+            inserted = run_id not in ctx["runs"]
+            run_ctx = ctx["runs"].setdefault(run_id, default_ctx())
+
+            r = run_ctx.setdefault("run", {})
+            r.setdefault("display_name", "lovely-dawn-{}".format(run_num + 32))
+            r.setdefault("storage_id", "storageid{}".format(run_num))
+            r.setdefault("project_name", "test")
+            r.setdefault("entity_name", "mock_server_entity")
+
+            param_config = body["variables"].get("config")
+            if param_config:
+                for c in ctx, run_ctx:
+                    c.setdefault("config", []).append(json.loads(param_config))
+
+            param_summary = body["variables"].get("summaryMetrics")
+            if param_summary:
+                for c in ctx, run_ctx:
+                    c.setdefault("summary", []).append(json.loads(param_summary))
+
+            for c in ctx, run_ctx:
+                c["upsert_bucket_count"] += 1
+
+            # Update run context
+            ctx["runs"][run_id] = run_ctx
+
+            # support legacy tests which pass resume
+            if ctx["resume"] is True:
+                inserted = False
+
             response = {
                 "data": {
                     "upsertBucket": {
                         "bucket": {
-                            "id": "storageid",
-                            "name": body["variables"].get("name", "abc123"),
-                            "displayName": "lovely-dawn-32",
+                            "id": r["storage_id"],
+                            "name": run_id,
+                            "displayName": r["display_name"],
                             "project": {
-                                "name": "test",
-                                "entity": {"name": "mock_server_entity"},
+                                "name": r["project_name"],
+                                "entity": {"name": r["entity_name"]},
                             },
                         },
-                        "inserted": ctx["resume"] is False,
+                        "inserted": inserted,
                     }
                 }
             }
@@ -749,7 +790,9 @@ def create_app(user_ctx=None):
                 }
             }
         if "query Artifact(" in body["query"]:
-            art = artifact(ctx, request_url_root=base_url)
+            art = artifact(
+                ctx, request_url_root=base_url, id_override="QXJ0aWZhY3Q6NTI1MDk4"
+            )
             if "id" in body.get("variables", {}):
                 art = artifact(
                     ctx,
@@ -782,6 +825,8 @@ def create_app(user_ctx=None):
                 },
             }
             return {"data": {"project": {"artifact": art}}}
+        if "query ClientIDMapping(" in body["query"]:
+            return {"data": {"clientIDMapping": {"serverID": "QXJ0aWZhY3Q6NTI1MDk4"}}}
         if "stopped" in body["query"]:
             return json.dumps(
                 {
@@ -909,6 +954,30 @@ def create_app(user_ctx=None):
                         }
                     },
                 }
+            elif _id == "e6954815d2beb5841b3dabf7cf455c30":
+                return {
+                    "version": 1,
+                    "storagePolicy": "wandb-storage-policy-v1",
+                    "storagePolicyConfig": {},
+                    "contents": {
+                        "logged_table.partitioned-table.json": {
+                            "digest": "3aaaaaaaaaaaaaaaaaaaaa==",
+                            "size": 81299,
+                        }
+                    },
+                }
+            elif _id == "0eec13efd400546f58a4530de62ed07a":
+                return {
+                    "version": 1,
+                    "storagePolicy": "wandb-storage-policy-v1",
+                    "storagePolicyConfig": {},
+                    "contents": {
+                        "jt.joined-table.json": {
+                            "digest": "3aaaaaaaaaaaaaaaaaaaaa==",
+                            "size": 81299,
+                        }
+                    },
+                }
             elif _id in [
                 "2d9a7e0aa8407f0730e19e5bc55c3a45",
                 "c541de19b18331a4a33b282fc9d42510",
@@ -1005,7 +1074,7 @@ index 30d74d2..9a2c773 100644
 
     @app.route("/artifacts/<entity>/<digest>", methods=["GET", "POST"])
     def artifact_file(entity, digest):
-        if entity == "entity":
+        if entity == "entity" or entity == "mock_server_entity":
             if (
                 digest == "d1a69a69a69a69a69a69a69a69a69a69"
             ):  # "dataset.partitioned-table.json"
@@ -1089,8 +1158,10 @@ index 30d74d2..9a2c773 100644
     @app.route("/files/<entity>/<project>/<run>/file_stream", methods=["POST"])
     def file_stream(entity, project, run):
         ctx = get_ctx()
-        ctx["file_stream"] = ctx.get("file_stream", [])
-        ctx["file_stream"].append(request.get_json())
+        run_ctx = get_run_ctx(run)
+        for c in ctx, run_ctx:
+            c["file_stream"] = c.get("file_stream", [])
+            c["file_stream"].append(request.get_json())
         response = json.dumps({"exitcode": None, "limits": {}})
 
         inject = InjectRequestsParse(ctx).find(request=request)
@@ -1166,12 +1237,13 @@ index 30d74d2..9a2c773 100644
 
 
 class ParseCTX(object):
-    def __init__(self, ctx):
-        self._ctx = ctx
+    def __init__(self, ctx, run_id=None):
+        self._ctx = ctx["runs"][run_id] if run_id else ctx
+        self._run_id = run_id
 
     def get_filestream_file_updates(self):
         data = {}
-        file_stream_updates = self._ctx["file_stream"]
+        file_stream_updates = self._ctx.get("file_stream", [])
         for update in file_stream_updates:
             files = update.get("files")
             if not files:
@@ -1190,7 +1262,9 @@ class ParseCTX(object):
                 content = d.get("content")
                 assert offset is not None
                 assert content is not None
-                assert offset == 0 or offset == len(l), (k, v, l, d)
+                # this check isnt valid right now.
+                # TODO: lets just assume it is fine, look into this later
+                # assert offset == 0 or offset == len(l), (k, v, l, d)
                 if not offset:
                     l = []
                 if k == u"output.log":
@@ -1202,21 +1276,55 @@ class ParseCTX(object):
         return data
 
     @property
-    def summary(self):
+    def runs(self):
+        return list(self._ctx.get("runs", {}))
+
+    @property
+    def dropped_chunks(self):
+        return self._ctx.get("file_stream", [{"dropped": 0}])[-1]["dropped"]
+
+    @property
+    def summary_raw(self):
         fs_files = self.get_filestream_file_items()
-        summary = fs_files["wandb-summary.json"][-1]
+        summary = fs_files.get("wandb-summary.json", [])[-1]
         return summary
 
+    @property
+    def summary_user(self):
+        return {k: v for k, v in self.summary_raw.items() if not k.startswith("_")}
+
+    @property
+    def summary(self):
+        # TODO: move this to config_user eventually
+        return self.summary_raw
+
     @property
     def history(self):
         fs_files = self.get_filestream_file_items()
-        history = fs_files["wandb-history.jsonl"]
+        history = fs_files.get("wandb-history.jsonl")
         return history
 
     @property
-    def config(self):
+    def exit_code(self):
+        exit_code = None
+        fs_list = self._ctx.get("file_stream")
+        if fs_list:
+            exit_code = fs_list[-1].get("exitcode")
+        return exit_code
+
+    @property
+    def config_raw(self):
         return self._ctx["config"][-1]
 
+    @property
+    def config_user(self):
+        return {k: v["value"] for k, v in self.config_raw.items() if not k.startswith("_")}
+
+    @property
+    def config(self):
+        # TODO: move this to config_user eventually
+        return self.config_raw
+
     @property
     def config_wandb(self):
         return self.config["_wandb"]["value"]
